package com.sun.jmx.mbeanserver;

import java.security.AccessController;
import java.security.AccessControlContext;
import java.security.ProtectionDomain;
import java.security.PrivilegedAction;
import java.security.Permissions;
import java.lang.reflect.Modifier;
import javax.management.MBeanPermission;
import sun.reflect.misc.ConstructorUtil;
import java.io.IOException;
import javax.management.OperationsException;
import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import javax.management.RuntimeMBeanException;
import javax.management.RuntimeErrorException;
import javax.management.MBeanException;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.InstanceNotFoundException;
import javax.management.ObjectName;
import javax.management.RuntimeOperationsException;
import javax.management.ReflectionException;
import sun.reflect.misc.ReflectUtil;
import javax.management.NotCompliantMBeanException;
import java.util.Map;

public class MBeanInstantiator
{
    private final ModifiableClassLoaderRepository clr;
    private static final Map<String, Class<?>> primitiveClasses;
    
    MBeanInstantiator(ModifiableClassLoaderRepository clr) {
        this.clr = clr;
    }
    
    public void testCreation(Class<?> clazz) throws NotCompliantMBeanException {
        Introspector.testCreation(clazz);
    }
    
    public Class<?> findClassWithDefaultLoaderRepository(String s) throws ReflectionException {
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("The class name cannot be null"), "Exception occurred during object instantiation");
        }
        ReflectUtil.checkPackageAccess(s);
        Class<Object> loadClass;
        try {
            if (this.clr == null) {
                throw new ClassNotFoundException(s);
            }
            loadClass = this.clr.loadClass(s);
        }
        catch (ClassNotFoundException e) {
            throw new ReflectionException(e, "The MBean class could not be loaded by the default loader repository");
        }
        return loadClass;
    }
    
    public Class<?> findClass(String s, ClassLoader classLoader) throws ReflectionException {
        return loadClass(s, classLoader);
    }
    
    public Class<?> findClass(String s, ObjectName obj) throws ReflectionException, InstanceNotFoundException {
        if (obj == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException(), "Null loader passed in parameter");
        }
        ClassLoader classLoader = null;
        synchronized (this) {
            classLoader = this.getClassLoader(obj);
        }
        if (classLoader == null) {
            throw new InstanceNotFoundException("The loader named " + obj + " is not registered in the MBeanServer");
        }
        return this.findClass(s, classLoader);
    }
    
    public Class<?>[] findSignatureClasses(String[] array, ClassLoader loader) throws ReflectionException {
        if (array == null) {
            return null;
        }
        int length = array.length;
        Class[] array2 = new Class[length];
        if (length == 0) {
            return array2;
        }
        try {
            for (int i = 0; i < length; ++i) {
                Class<Object> clazz = (Class)MBeanInstantiator.primitiveClasses.get(array[i]);
                if (clazz != null) {
                    array2[i] = clazz;
                }
                else {
                    ReflectUtil.checkPackageAccess(array[i]);
                    if (loader != null) {
                        array2[i] = Class.forName(array[i], false, loader);
                    }
                    else {
                        array2[i] = this.findClass(array[i], this.getClass().getClassLoader());
                    }
                }
            }
        }
        catch (ClassNotFoundException ex) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, MBeanInstantiator.class.getName(), "findSignatureClasses", "The parameter class could not be found", ex);
            }
            throw new ReflectionException(ex, "The parameter class could not be found");
        }
        catch (RuntimeException thrown) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, MBeanInstantiator.class.getName(), "findSignatureClasses", "Unexpected exception", thrown);
            }
            throw thrown;
        }
        return array2;
    }
    
    public Object instantiate(Class<?> clazz) throws ReflectionException, MBeanException {
        checkMBeanPermission(clazz, null, null, "instantiate");
        Constructor<Object> constructor = this.findConstructor(clazz, (Class[])null);
        if (constructor == null) {
            throw new ReflectionException(new NoSuchMethodException("No such constructor"));
        }
        Object instance;
        try {
            ReflectUtil.checkPackageAccess(clazz);
            ensureClassAccess(clazz);
            instance = constructor.newInstance(new Object[0]);
        }
        catch (InvocationTargetException ex) {
            Throwable targetException = ex.getTargetException();
            if ((targetException instanceof RuntimeException)) {
                throw new RuntimeMBeanException((RuntimeException)targetException, "RuntimeException thrown in the MBean's empty constructor");
            }
            if ((targetException instanceof Error)) {
                throw new RuntimeErrorException((Error)targetException, "Error thrown in the MBean's empty constructor");
            }
            throw new MBeanException((Exception)targetException, "Exception thrown in the MBean's empty constructor");
        }
        catch (NoSuchMethodError noSuchMethodError) {
            throw new ReflectionException(new NoSuchMethodException("No constructor"), "No such constructor");
        }
        catch (InstantiationException e) {
            throw new ReflectionException(e, "Exception thrown trying to invoke the MBean's empty constructor");
        }
        catch (IllegalAccessException e2) {
            throw new ReflectionException(e2, "Exception thrown trying to invoke the MBean's empty constructor");
        }
        catch (IllegalArgumentException e3) {
            throw new ReflectionException(e3, "Exception thrown trying to invoke the MBean's empty constructor");
        }
        return instance;
    }
    
    public Object instantiate(Class<?> clazz, Object[] initargs, String[] array, ClassLoader classLoader) throws ReflectionException, MBeanException {
        checkMBeanPermission(clazz, null, null, "instantiate");
        Class[] array2;
        try {
            ClassLoader classLoader2 = clazz.getClassLoader();
            array2 = ((array == null) ? null : this.findSignatureClasses(array, classLoader2));
        }
        catch (IllegalArgumentException e) {
            throw new ReflectionException(e, "The constructor parameter classes could not be loaded");
        }
        Constructor<Object> constructor = this.findConstructor(clazz, array2);
        if (constructor == null) {
            throw new ReflectionException(new NoSuchMethodException("No such constructor"));
        }
        Object instance;
        try {
            ReflectUtil.checkPackageAccess(clazz);
            ensureClassAccess(clazz);
            instance = constructor.newInstance(initargs);
        }
        catch (NoSuchMethodError noSuchMethodError) {
            throw new ReflectionException(new NoSuchMethodException("No such constructor found"), "No such constructor");
        }
        catch (InstantiationException e2) {
            throw new ReflectionException(e2, "Exception thrown trying to invoke the MBean's constructor");
        }
        catch (IllegalAccessException e3) {
            throw new ReflectionException(e3, "Exception thrown trying to invoke the MBean's constructor");
        }
        catch (InvocationTargetException ex) {
            Throwable targetException = ex.getTargetException();
            if ((targetException instanceof RuntimeException)) {
                throw new RuntimeMBeanException((RuntimeException)targetException, "RuntimeException thrown in the MBean's constructor");
            }
            if ((targetException instanceof Error)) {
                throw new RuntimeErrorException((Error)targetException, "Error thrown in the MBean's constructor");
            }
            throw new MBeanException((Exception)targetException, "Exception thrown in the MBean's constructor");
        }
        return instance;
    }
    
    public ObjectInputStream deserialize(ClassLoader classLoader, byte[] buf) throws OperationsException {
        if (buf == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException(), "Null data passed in parameter");
        }
        if (buf.length == 0) {
            throw new RuntimeOperationsException(new IllegalArgumentException(), "Empty data passed in parameter");
        }
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(buf);
        ObjectInputStreamWithLoader objectInputStreamWithLoader;
        try {
            objectInputStreamWithLoader = new ObjectInputStreamWithLoader(byteArrayInputStream, classLoader);
        }
        catch (IOException ex) {
            throw new OperationsException("An IOException occurred trying to de-serialize the data");
        }
        return objectInputStreamWithLoader;
    }
    
    public ObjectInputStream deserialize(String s, ObjectName objectName, byte[] buf, ClassLoader classLoader) throws InstanceNotFoundException, OperationsException, ReflectionException {
        if (buf == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException(), "Null data passed in parameter");
        }
        if (buf.length == 0) {
            throw new RuntimeOperationsException(new IllegalArgumentException(), "Empty data passed in parameter");
        }
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException(), "Null className passed in parameter");
        }
        ReflectUtil.checkPackageAccess(s);
        Class<Object> clazz;
        if (objectName == null) {
            clazz = this.findClass(s, classLoader);
        }
        else {
            try {
                ClassLoader classLoader2 = this.getClassLoader(objectName);
                if (classLoader2 == null) {
                    throw new ClassNotFoundException(s);
                }
                clazz = Class.forName(s, false, classLoader2);
            }
            catch (ClassNotFoundException e) {
                throw new ReflectionException(e, "The MBean class could not be loaded by the " + objectName.toString() + " class loader");
            }
        }
        ByteArrayInputStream o = new ByteArrayInputStream(buf);
        Object o2;
        try {
            o2 = new ObjectInputStreamWithLoader(o, clazz.getClassLoader());
        }
        catch (IOException ex) {
            throw new OperationsException("An IOException occurred trying to de-serialize the data");
        }
        return o2;
    }
    
    public Object instantiate(String s) throws ReflectionException, MBeanException {
        return this.instantiate(s, (Object[])null, (String[])null, null);
    }
    
    public Object instantiate(String s, ObjectName objectName, ClassLoader classLoader) throws ReflectionException, MBeanException, InstanceNotFoundException {
        return this.instantiate(s, objectName, (Object[])null, (String[])null, classLoader);
    }
    
    public Object instantiate(String s, Object[] array, String[] array2, ClassLoader classLoader) throws ReflectionException, MBeanException {
        return this.instantiate(this.findClassWithDefaultLoaderRepository(s), array, array2, classLoader);
    }
    
    public Object instantiate(String s, ObjectName objectName, Object[] array, String[] array2, ClassLoader classLoader) throws ReflectionException, MBeanException, InstanceNotFoundException {
        Class<Object> clazz;
        if (objectName == null) {
            clazz = this.findClass(s, classLoader);
        }
        else {
            clazz = this.findClass(s, objectName);
        }
        return this.instantiate(clazz, array, array2, classLoader);
    }
    
    public ModifiableClassLoaderRepository getClassLoaderRepository() {
        checkMBeanPermission((String)null, null, null, "getClassLoaderRepository");
        return this.clr;
    }
    
    static Class<?> loadClass(String name, ClassLoader classLoader) throws ReflectionException {
        if (name == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("The class name cannot be null"), "Exception occurred during object instantiation");
        }
        ReflectUtil.checkPackageAccess(name);
        Class<Object> clazz;
        try {
            if (classLoader == null) {
                classLoader = MBeanInstantiator.class.getClassLoader();
            }
            if (classLoader != null) {
                clazz = Class.forName(name, false, classLoader);
            }
            else {
                clazz = Class.forName(name);
            }
        }
        catch (ClassNotFoundException e) {
            throw new ReflectionException(e, "The MBean class could not be loaded");
        }
        return clazz;
    }
    
    static Class<?>[] loadSignatureClasses(String[] array, ClassLoader loader) throws ReflectionException {
        if (array == null) {
            return null;
        }
        loader = ((loader == null) ? MBeanInstantiator.class.getClassLoader() : loader);
        int length = array.length;
        Class[] array2 = new Class[length];
        if (length == 0) {
            return array2;
        }
        try {
            for (int i = 0; i < length; ++i) {
                Class<Object> clazz = (Class)MBeanInstantiator.primitiveClasses.get(array[i]);
                if (clazz != null) {
                    array2[i] = clazz;
                }
                else {
                    ReflectUtil.checkPackageAccess(array[i]);
                    array2[i] = Class.forName(array[i], false, loader);
                }
            }
        }
        catch (ClassNotFoundException ex) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, MBeanInstantiator.class.getName(), "findSignatureClasses", "The parameter class could not be found", ex);
            }
            throw new ReflectionException(ex, "The parameter class could not be found");
        }
        catch (RuntimeException thrown) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, MBeanInstantiator.class.getName(), "findSignatureClasses", "Unexpected exception", thrown);
            }
            throw thrown;
        }
        return array2;
    }
    
    private Constructor<?> findConstructor(Class<?> cls, Class<?>[] params) {
        try {
            return ConstructorUtil.getConstructor(cls, params);
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    private static void checkMBeanPermission(Class<?> clazz, String s, ObjectName objectName, String s2) {
        if (clazz != null) {
            checkMBeanPermission(clazz.getName(), s, objectName, s2);
        }
    }
    
    private static void checkMBeanPermission(String className, String member, ObjectName objectName, String actions) throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new MBeanPermission(className, member, objectName, actions));
        }
    }
    
    private static void ensureClassAccess(Class clazz) throws IllegalAccessException {
        if (!Modifier.isPublic(clazz.getModifiers())) {
            throw new IllegalAccessException("Class is not public and can't be instantiated");
        }
        return;
    }
    
    private ClassLoader getClassLoader(final ObjectName objectName) {
        if (this.clr == null) {
            return null;
        }
        Permissions permissions = new Permissions();
        permissions.add(new MBeanPermission("*", null, objectName, "getClassLoader"));
        return (ClassLoader)AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
            @Override
            public ClassLoader run() {
                return MBeanInstantiator.this.clr.getClassLoader(objectName);
            }
        }, new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, permissions) }));
    }
    
    static {
        primitiveClasses = Util.newMap();
        for (Class clazz : new Class[] { Byte.TYPE, Short.TYPE, Integer.TYPE, Long.TYPE, Float.TYPE, Double.TYPE, Character.TYPE, Boolean.TYPE }) {
            MBeanInstantiator.primitiveClasses.put(clazz.getName(), clazz);
        }
    }
}
